@@ -21,12 +21,27 @@
 import abc
 
 from pyalgotrade import observer
+from pyalgotrade import warninghelpers
+
+
+# This class is used to prevent bugs like the one triggered in testcases.bitstamp_test:TestCase.testRoundingBug.
+# Why not use decimal.Decimal instead ?
+# 1: I'd have to expose this to users. They'd have to deal with decimal.Decimal and it'll break existing users.
+# 2: numpy arrays built using decimal.Decimal instances have dtype=object.
+class InstrumentTraits(object):
+
+    __metaclass__ = abc.ABCMeta
+
+    # Return the floating point value number rounded.
+    @abc.abstractmethod
+    def roundQuantity(self, quantity):
+        raise NotImplementedError()
 
 
 ######################################################################
-## Orders
-## http://stocks.about.com/od/tradingbasics/a/markords.htm
-## http://www.interactivebrokers.com/en/software/tws/usersguidebook/ordertypes/basic_order_types.htm
+# Orders
+# http://stocks.about.com/od/tradingbasics/a/markords.htm
+# http://www.interactivebrokers.com/en/software/tws/usersguidebook/ordertypes/basic_order_types.htm
 #
 # State chart:
 # INITIAL           -> SUBMITTED
@@ -43,8 +58,6 @@ from pyalgotrade import observer
 class Order(object):
     """Base class for orders.
 
-    :param orderId: The order id.
-    :type orderId: string.
     :param type_: The order type
     :type type_: :class:`Order.Type`
     :param action: The order action.
@@ -117,14 +130,15 @@ class Order(object):
         State.PARTIALLY_FILLED: [State.PARTIALLY_FILLED, State.FILLED, State.CANCELED],
     }
 
-    def __init__(self, orderId, type_, action, instrument, quantity):
+    def __init__(self, type_, action, instrument, quantity, instrumentTraits):
         if quantity <= 0:
             raise Exception("Invalid quantity")
-        self.__id = orderId
+        self.__id = None
         self.__type = type_
         self.__action = action
         self.__instrument = instrument
         self.__quantity = quantity
+        self.__instrumentTraits = instrumentTraits
         self.__filled = 0
         self.__avgFillPrice = None
         self.__executionInfo = None
@@ -132,21 +146,31 @@ class Order(object):
         self.__commissions = 0
         self.__allOrNone = False
         self.__state = Order.State.INITIAL
+        self.__submitDateTime = None
 
     # This is to check that orders are not compared directly. order ids should be compared.
-    #def __eq__(self, other):
-    #    if other is None:
-    #        return False
-    #    assert(False)
+#    def __eq__(self, other):
+#        if other is None:
+#            return False
+#        assert(False)
 
     # This is to check that orders are not compared directly. order ids should be compared.
-    #def __ne__(self, other):
-    #    if other is None:
-    #        return True
-    #    assert(False)
+#    def __ne__(self, other):
+#        if other is None:
+#            return True
+#        assert(False)
+
+    def getInstrumentTraits(self):
+        return self.__instrumentTraits
 
     def getId(self):
-        """Returns the order id."""
+        """
+        Returns the order id.
+
+        .. note::
+
+            This will be None if the order was not submitted.
+        """
         return self.__id
 
     def getType(self):
@@ -159,6 +183,15 @@ class Order(object):
         """
         return self.__type
 
+    def getSubmitDateTime(self):
+        """Returns the datetime when the order was submitted."""
+        return self.__submitDateTime
+
+    def setSubmitted(self, orderId, dateTime):
+        assert(self.__id is None or orderId == self.__id)
+        self.__id = orderId
+        self.__submitDateTime = dateTime
+
     def getAction(self):
         """Returns the order action. Valid order actions are:
 
@@ -223,7 +256,7 @@ class Order(object):
 
     def getRemaining(self):
         """Returns the number of shares still outstanding."""
-        return self.__quantity - self.__filled
+        return self.__instrumentTraits.roundQuantity(self.__quantity - self.__filled)
 
     def getAvgFillPrice(self):
         """Returns the average price of the shares that have been executed, or None if nothing has been filled."""
@@ -276,7 +309,7 @@ class Order(object):
             self.__avgFillPrice = (self.__avgFillPrice * self.__filled + orderExecutionInfo.getPrice() * orderExecutionInfo.getQuantity()) / float(self.__filled + orderExecutionInfo.getQuantity())
 
         self.__executionInfo = orderExecutionInfo
-        self.__filled += orderExecutionInfo.getQuantity()
+        self.__filled = self.getInstrumentTraits().roundQuantity(self.__filled + orderExecutionInfo.getQuantity())
         self.__commissions += orderExecutionInfo.getCommission()
 
         if self.getRemaining() == 0:
@@ -320,8 +353,8 @@ class MarketOrder(Order):
         This is a base class and should not be used directly.
     """
 
-    def __init__(self, orderId, action, instrument, quantity, onClose):
-        Order.__init__(self, orderId, Order.Type.MARKET, action, instrument, quantity)
+    def __init__(self, action, instrument, quantity, onClose, instrumentTraits):
+        Order.__init__(self, Order.Type.MARKET, action, instrument, quantity, instrumentTraits)
         self.__onClose = onClose
 
     def getFillOnClose(self):
@@ -337,8 +370,8 @@ class LimitOrder(Order):
         This is a base class and should not be used directly.
     """
 
-    def __init__(self, orderId, action, instrument, limitPrice, quantity):
-        Order.__init__(self, orderId, Order.Type.LIMIT, action, instrument, quantity)
+    def __init__(self, action, instrument, limitPrice, quantity, instrumentTraits):
+        Order.__init__(self, Order.Type.LIMIT, action, instrument, quantity, instrumentTraits)
         self.__limitPrice = limitPrice
 
     def getLimitPrice(self):
@@ -354,8 +387,8 @@ class StopOrder(Order):
         This is a base class and should not be used directly.
     """
 
-    def __init__(self, orderId, action, instrument, stopPrice, quantity):
-        Order.__init__(self, orderId, Order.Type.STOP, action, instrument, quantity)
+    def __init__(self, action, instrument, stopPrice, quantity, instrumentTraits):
+        Order.__init__(self, Order.Type.STOP, action, instrument, quantity, instrumentTraits)
         self.__stopPrice = stopPrice
 
     def getStopPrice(self):
@@ -371,8 +404,8 @@ class StopLimitOrder(Order):
         This is a base class and should not be used directly.
     """
 
-    def __init__(self, orderId, action, instrument, stopPrice, limitPrice, quantity):
-        Order.__init__(self, orderId, Order.Type.STOP_LIMIT, action, instrument, quantity)
+    def __init__(self, action, instrument, stopPrice, limitPrice, quantity, instrumentTraits):
+        Order.__init__(self, Order.Type.STOP_LIMIT, action, instrument, quantity, instrumentTraits)
         self.__stopPrice = stopPrice
         self.__limitPrice = limitPrice
 
@@ -393,6 +426,9 @@ class OrderExecutionInfo(object):
         self.__commission = commission
         self.__dateTime = dateTime
 
+    def __str__(self):
+        return "%s - Price: %s - Amount: %s - Fee: %s" % (self.__dateTime, self.__price, self.__quantity, self.__commission)
+
     def getPrice(self):
         """Returns the fill price."""
         return self.__price
@@ -438,7 +474,7 @@ class OrderEvent(object):
 
 
 ######################################################################
-## Base broker class
+# Base broker class
 class Broker(observer.Subject):
     """Base class for brokers.
 
@@ -461,6 +497,20 @@ class Broker(observer.Subject):
     def getOrderUpdatedEvent(self):
         return self.__orderEvent
 
+    @abc.abstractmethod
+    def getInstrumentTraits(self, instrument):
+        raise NotImplementedError()
+
+    @abc.abstractmethod
+    def getCash(self, includeShort=True):
+        """
+        Returns the available cash.
+
+        :param includeShort: Include cash from short positions.
+        :type includeShort: boolean.
+        """
+        raise NotImplementedError()
+
     @abc.abstractmethod
     def getShares(self, instrument):
         """Returns the number of shares for an instrument."""
@@ -481,7 +531,7 @@ class Broker(observer.Subject):
         raise NotImplementedError()
 
     @abc.abstractmethod
-    def placeOrder(self, order):
+    def submitOrder(self, order):
         """Submits an order.
 
         :param order: The order to submit.
@@ -493,6 +543,11 @@ class Broker(observer.Subject):
         """
         raise NotImplementedError()
 
+    def placeOrder(self, order):
+        # Deprecated since v0.16
+        warninghelpers.deprecation_warning("placeOrder will be deprecated in the next version. Please use submitOrder instead.", stacklevel=2)
+        return self.submitOrder(order)
+
     @abc.abstractmethod
     def createMarketOrder(self, action, instrument, quantity, onClose=False):
         """Creates a Market order.
