@@ -26,8 +26,13 @@ from pyalgotrade import logger
 import pyalgotrade.bar
 
 
+class DefaultTraits(broker.InstrumentTraits):
+    def roundQuantity(self, quantity):
+        return int(quantity)
+
+
 ######################################################################
-## Commission models
+# Commission models
 
 class Commission(object):
     """Base class for implementing different commission schemes.
@@ -92,7 +97,7 @@ class TradePercentage(Commission):
 
 
 ######################################################################
-## Order filling strategies
+# Order filling strategies
 
 # Returns the trigger price for a Stop or StopLimit order, or None if the stop price was not yet penetrated.
 def get_stop_price_trigger(action, stopPrice, useAdjustedValues, bar):
@@ -291,10 +296,13 @@ class DefaultStrategy(FillStrategy):
                 volumeLeft[instrument] = bar.getVolume() * self.__volumeLimit
         self.__volumeLeft = volumeLeft
 
+    def getVolumeLeft(self):
+        return self.__volumeLeft
+
     def onOrderFilled(self, order):
         # Update the volume left.
         if self.__volumeLimit is not None:
-            self.__volumeLeft[order.getInstrument()] -= order.getExecutionInfo().getQuantity()
+            self.__volumeLeft[order.getInstrument()] = order.getInstrumentTraits().roundQuantity(self.__volumeLeft[order.getInstrument()] - order.getExecutionInfo().getQuantity())
 
     def setVolumeLimit(self, volumeLimit):
         self.__volumeLimit = volumeLimit
@@ -302,14 +310,13 @@ class DefaultStrategy(FillStrategy):
     def __calculateFillSize(self, order, broker_, bar):
         ret = 0
 
-        # If self.__volumeLimit then allow all the order to get filled.
+        # If self.__volumeLimit is None then allow all the order to get filled.
         if self.__volumeLimit is not None:
             volumeLeft = self.__volumeLeft.get(order.getInstrument(), 0)
         else:
             volumeLeft = order.getRemaining()
 
-        if not broker_.getAllowFractions():
-            volumeLeft = int(volumeLeft)
+        volumeLeft = order.getInstrumentTraits().roundQuantity(volumeLeft)
 
         if not order.getAllOrNone():
             ret = min(volumeLeft, order.getRemaining())
@@ -321,6 +328,7 @@ class DefaultStrategy(FillStrategy):
     def fillMarketOrder(self, order, broker_, bar):
         fillSize = self.__calculateFillSize(order, broker_, bar)
         if fillSize == 0:
+            broker_.getLogger().debug("Not enough volume to fill %s market order [%s] for %d share/s" % (order.getInstrument(), order.getId(), order.getRemaining()))
             return None
 
         ret = None
@@ -335,6 +343,7 @@ class DefaultStrategy(FillStrategy):
     def fillLimitOrder(self, order, broker_, bar):
         fillSize = self.__calculateFillSize(order, broker_, bar)
         if fillSize == 0:
+            broker_.getLogger().debug("Not enough volume to fill %s limit order [%s] for %d share/s" % (order.getInstrument(), order.getId(), order.getRemaining()))
             return None
 
         ret = None
@@ -356,6 +365,7 @@ class DefaultStrategy(FillStrategy):
         if order.getStopHit():
             fillSize = self.__calculateFillSize(order, broker_, bar)
             if fillSize == 0:
+                broker_.getLogger().debug("Not enough volume to fill %s stop order [%s] for %d share/s" % (order.getInstrument(), order.getId(), order.getRemaining()))
                 return None
 
             # If we just hit the stop price we'll use it as the fill price.
@@ -381,6 +391,7 @@ class DefaultStrategy(FillStrategy):
         if order.getStopHit():
             fillSize = self.__calculateFillSize(order, broker_, bar)
             if fillSize == 0:
+                broker_.getLogger().debug("Not enough volume to fill %s stop limit order [%s] for %d share/s" % (order.getInstrument(), order.getId(), order.getRemaining()))
                 return None
 
             price = get_limit_price_trigger(order.getAction(), order.getLimitPrice(), broker_.getUseAdjustedValues(), bar)
@@ -400,7 +411,7 @@ class DefaultStrategy(FillStrategy):
 
 
 ######################################################################
-## Orders
+# Orders
 
 class BacktestingOrder(object):
     def __init__(self):
@@ -419,8 +430,8 @@ class BacktestingOrder(object):
 
 
 class MarketOrder(broker.MarketOrder, BacktestingOrder):
-    def __init__(self, orderId, action, instrument, quantity, onClose):
-        broker.MarketOrder.__init__(self, orderId, action, instrument, quantity, onClose)
+    def __init__(self, action, instrument, quantity, onClose, instrumentTraits):
+        broker.MarketOrder.__init__(self, action, instrument, quantity, onClose, instrumentTraits)
         BacktestingOrder.__init__(self)
 
     def process(self, broker_, bar_):
@@ -428,8 +439,8 @@ class MarketOrder(broker.MarketOrder, BacktestingOrder):
 
 
 class LimitOrder(broker.LimitOrder, BacktestingOrder):
-    def __init__(self, orderId, action, instrument, limitPrice, quantity):
-        broker.LimitOrder.__init__(self, orderId, action, instrument, limitPrice, quantity)
+    def __init__(self, action, instrument, limitPrice, quantity, instrumentTraits):
+        broker.LimitOrder.__init__(self, action, instrument, limitPrice, quantity, instrumentTraits)
         BacktestingOrder.__init__(self)
 
     def process(self, broker_, bar_):
@@ -437,8 +448,8 @@ class LimitOrder(broker.LimitOrder, BacktestingOrder):
 
 
 class StopOrder(broker.StopOrder, BacktestingOrder):
-    def __init__(self, orderId, action, instrument, stopPrice, quantity):
-        broker.StopOrder.__init__(self, orderId, action, instrument, stopPrice, quantity)
+    def __init__(self, action, instrument, stopPrice, quantity, instrumentTraits):
+        broker.StopOrder.__init__(self, action, instrument, stopPrice, quantity, instrumentTraits)
         BacktestingOrder.__init__(self)
         self.__stopHit = False
 
@@ -455,8 +466,8 @@ class StopOrder(broker.StopOrder, BacktestingOrder):
 # http://www.sec.gov/answers/stoplim.htm
 # http://www.interactivebrokers.com/en/trading/orders/stopLimit.php
 class StopLimitOrder(broker.StopLimitOrder, BacktestingOrder):
-    def __init__(self, orderId, action, instrument, stopPrice, limitPrice, quantity):
-        broker.StopLimitOrder.__init__(self, orderId, action, instrument, stopPrice, limitPrice, quantity)
+    def __init__(self, action, instrument, stopPrice, limitPrice, quantity, instrumentTraits):
+        broker.StopLimitOrder.__init__(self, action, instrument, stopPrice, limitPrice, quantity, instrumentTraits)
         BacktestingOrder.__init__(self)
         self.__stopHit = False  # Set to true when the limit order is activated (stop price is hit)
 
@@ -475,7 +486,7 @@ class StopLimitOrder(broker.StopLimitOrder, BacktestingOrder):
 
 
 ######################################################################
-## Broker
+# Broker
 
 class Broker(broker.Broker):
     """Backtesting broker.
@@ -503,7 +514,6 @@ class Broker(broker.Broker):
         self.__activeOrders = {}
         self.__useAdjustedValues = False
         self.__fillStrategy = DefaultStrategy()
-        self.__allowFractions = False
         self.__logger = logger.getLogger(Broker.LOGGER_NAME)
 
         # It is VERY important that the broker subscribes to barfeed events before the strategy.
@@ -512,42 +522,40 @@ class Broker(broker.Broker):
         self.__allowNegativeCash = False
         self.__nextOrderId = 1
 
-    def __getNextOrderId(self):
+    def _getNextOrderId(self):
         ret = self.__nextOrderId
         self.__nextOrderId += 1
         return ret
 
-    def __getBar(self, bars, instrument):
+    def _getBar(self, bars, instrument):
         ret = bars.getBar(instrument)
         if ret is None:
             ret = self.__barFeed.getLastBar(instrument)
         return ret
 
+    def _registerOrder(self, order):
+        assert(order.getId() not in self.__activeOrders)
+        assert(order.getId() is not None)
+        self.__activeOrders[order.getId()] = order
+
+    def _unregisterOrder(self, order):
+        assert(order.getId() in self.__activeOrders)
+        assert(order.getId() is not None)
+        del self.__activeOrders[order.getId()]
+
     def getLogger(self):
         return self.__logger
 
-    def setAllowFractions(self, allowFractions):
-        self.__allowFractions = allowFractions
-
-    def getAllowFractions(self):
-        return self.__allowFractions
-
     def setAllowNegativeCash(self, allowNegativeCash):
         self.__allowNegativeCash = allowNegativeCash
 
     def getCash(self, includeShort=True):
-        """
-        Returns the available cash.
-
-        :param includeShort: Include cash from short positions.
-        :type includeShort: boolean.
-        """
         ret = self.__cash
         if not includeShort and self.__barFeed.getCurrentBars() is not None:
             bars = self.__barFeed.getCurrentBars()
             for instrument, shares in self.__shares.iteritems():
                 if shares < 0:
-                    instrumentPrice = self.__getBar(bars, instrument).getClose(self.getUseAdjustedValues())
+                    instrumentPrice = self._getBar(bars, instrument).getClose(self.getUseAdjustedValues())
                     ret += instrumentPrice * shares
         return ret
 
@@ -600,6 +608,12 @@ class Broker(broker.Broker):
         warninghelpers.deprecation_warning("getPendingOrders will be deprecated in the next version. Please use getActiveOrders instead.", stacklevel=2)
         return self.getActiveOrders()
 
+    def _getCurrentDateTime(self):
+        return self.__barFeed.getCurrentDateTime()
+
+    def getInstrumentTraits(self, instrument):
+        return DefaultTraits()
+
     def getShares(self, instrument):
         return self.__shares.get(instrument, 0)
 
@@ -613,7 +627,7 @@ class Broker(broker.Broker):
         ret = self.getCash()
         if bars is not None:
             for instrument, shares in self.__shares.iteritems():
-                instrumentPrice = self.__getBar(bars, instrument).getClose(self.getUseAdjustedValues())
+                instrumentPrice = self._getBar(bars, instrument).getClose(self.getUseAdjustedValues())
                 ret += instrumentPrice * shares
         return ret
 
@@ -657,26 +671,30 @@ class Broker(broker.Broker):
 
             # Commit the order execution.
             self.__cash = resultingCash
-            self.__shares[order.getInstrument()] = self.getShares(order.getInstrument()) + sharesDelta
+            updatedShares = order.getInstrumentTraits().roundQuantity(self.getShares(order.getInstrument()) + sharesDelta)
+            if updatedShares == 0:
+                del self.__shares[order.getInstrument()]
+            else:
+                self.__shares[order.getInstrument()] = updatedShares
 
             # Let the strategy know that the order was filled.
             self.__fillStrategy.onOrderFilled(order)
 
             # Notify the order update
             if order.isFilled():
-                del self.__activeOrders[order.getId()]
+                self._unregisterOrder(order)
                 self.notifyOrderEvent(broker.OrderEvent(order, broker.OrderEvent.Type.FILLED, orderExecutionInfo))
             elif order.isPartiallyFilled():
                 self.notifyOrderEvent(broker.OrderEvent(order, broker.OrderEvent.Type.PARTIALLY_FILLED, orderExecutionInfo))
             else:
                 assert(False)
         else:
-            self.__logger.debug("Not enough money to fill order %s" % (order))
+            self.__logger.debug("Not enough cash to fill %s order [%s] for %d share/s" % (order.getInstrument(), order.getId(), order.getRemaining()))
 
-    def placeOrder(self, order):
+    def submitOrder(self, order):
         if order.isInitial():
-            # assert(order.getId() not in self.__activeOrders)
-            self.__activeOrders[order.getId()] = order
+            order.setSubmitted(self._getNextOrderId(), self._getCurrentDateTime())
+            self._registerOrder(order)
             # Switch from INITIAL -> SUBMITTED
             # IMPORTANT: Do not emit an event for this switch because when using the position interface
             # the order is not yet mapped to the position and Position.onOrderUpdated will get called.
@@ -695,7 +713,7 @@ class Broker(broker.Broker):
             # Cancel the order if it is expired.
             if expired:
                 ret = False
-                del self.__activeOrders[order.getId()]
+                self._unregisterOrder(order)
                 order.switchState(broker.Order.State.CANCELED)
                 self.notifyOrderEvent(broker.OrderEvent(order, broker.OrderEvent.Type.CANCELED, "Expired"))
 
@@ -711,7 +729,7 @@ class Broker(broker.Broker):
 
             # Cancel the order if it will expire in the next bar.
             if expired:
-                del self.__activeOrders[order.getId()]
+                self._unregisterOrder(order)
                 order.switchState(broker.Order.State.CANCELED)
                 self.notifyOrderEvent(broker.OrderEvent(order, broker.OrderEvent.Type.CANCELED, "Expired"))
 
@@ -780,16 +798,16 @@ class Broker(broker.Broker):
         return None
 
     def createMarketOrder(self, action, instrument, quantity, onClose=False):
-        return MarketOrder(self.__getNextOrderId(), action, instrument, quantity, onClose)
+        return MarketOrder(action, instrument, quantity, onClose, self.getInstrumentTraits(instrument))
 
     def createLimitOrder(self, action, instrument, limitPrice, quantity):
-        return LimitOrder(self.__getNextOrderId(), action, instrument, limitPrice, quantity)
+        return LimitOrder(action, instrument, limitPrice, quantity, self.getInstrumentTraits(instrument))
 
     def createStopOrder(self, action, instrument, stopPrice, quantity):
-        return StopOrder(self.__getNextOrderId(), action, instrument, stopPrice, quantity)
+        return StopOrder(action, instrument, stopPrice, quantity, self.getInstrumentTraits(instrument))
 
     def createStopLimitOrder(self, action, instrument, stopPrice, limitPrice, quantity):
-        return StopLimitOrder(self.__getNextOrderId(), action, instrument, stopPrice, limitPrice, quantity)
+        return StopLimitOrder(action, instrument, stopPrice, limitPrice, quantity, self.getInstrumentTraits(instrument))
 
     def cancelOrder(self, order):
         activeOrder = self.__activeOrders.get(order.getId())
@@ -798,6 +816,6 @@ class Broker(broker.Broker):
         if activeOrder.isFilled():
             raise Exception("Can't cancel order that has already been filled")
 
-        del self.__activeOrders[activeOrder.getId()]
+        self._unregisterOrder(activeOrder)
         activeOrder.switchState(broker.Order.State.CANCELED)
         self.notifyOrderEvent(broker.OrderEvent(activeOrder, broker.OrderEvent.Type.CANCELED, "User requested cancellation"))
