@@ -51,10 +51,11 @@ class BaseFeed(observer.Subject):
         self.__event = observer.Event()
         self.__maxLen = maxLen
 
-    # Return True if this is a real-time feed.
-    @abc.abstractmethod
-    def isRealTime(self):
-        raise NotImplementedError()
+    def reset(self):
+        keys = self.__ds.keys()
+        self.__ds = {}
+        for key in keys:
+            self.registerDataSeries(key)
 
     # Subclasses should implement this and return the appropriate dataseries for the given key.
     @abc.abstractmethod
