@@ -28,6 +28,10 @@ class MemFeed(feed.BaseFeed):
         self.__values = []
         self.__nextIdx = 0
 
+    def reset(self):
+        self.__nextIdx = 0
+        feed.BaseFeed.reset(self)
+
     def start(self):
         pass
 
@@ -49,9 +53,6 @@ class MemFeed(feed.BaseFeed):
             ret = self.__values[self.__nextIdx][0]
         return ret
 
-    def isRealTime(self):
-        return False
-
     def createDataSeries(self, key, maxLen):
         return dataseries.SequenceDataSeries(maxLen)
 
