@@ -139,7 +139,7 @@ class InstrumentMarker(Series):
     def __init__(self):
         Series.__init__(self)
         self.__useCandleSticks = False
-        self.__useAdjClose = False
+        self.__useAdjClose = None
         self.__marker = " "
 
     def needColor(self):
@@ -152,13 +152,16 @@ class InstrumentMarker(Series):
         return self.__marker
 
     def setUseAdjClose(self, useAdjClose):
+        # Force close/adj_close instead of price.
         self.__useAdjClose = useAdjClose
 
     def getValue(self, dateTime):
         # If not using candlesticks, the return the closing price.
         ret = Series.getValue(self, dateTime)
         if not self.__useCandleSticks and ret is not None:
-            if self.__useAdjClose:
+            if self.__useAdjClose is None:
+                ret = ret.getPrice()
+            elif self.__useAdjClose:
                 ret = ret.getAdjClose()
             else:
                 ret = ret.getClose()
@@ -226,7 +229,7 @@ class Subplot(object):
         return len(self.__series) == 0
 
     def addDataSeries(self, label, dataSeries, defaultClass=LineMarker):
-        """Adds a DataSeries to the subplot.
+        """Add a DataSeries to the subplot.
 
         :param label: A name for the DataSeries values.
         :type label: string.
@@ -237,7 +240,7 @@ class Subplot(object):
         self.__callbacks[callback] = self.getSeries(label, defaultClass)
 
     def addCallback(self, label, callback, defaultClass=LineMarker):
-        """Adds a callback that will be called on each bar.
+        """Add a callback that will be called on each bar.
 
         :param label: A name for the series values.
         :type label: string.
@@ -245,6 +248,16 @@ class Subplot(object):
         """
         self.__callbacks[callback] = self.getSeries(label, defaultClass)
 
+    def addLine(self, label, level):
+        """Add a horizontal line to the plot.
+
+        :param label: A label.
+        :type label: string.
+        :param level: The position for the line.
+        :type level: int/float.
+        """
+        self.addCallback(label, lambda x: level)
+
     def onBars(self, bars):
         dateTime = bars.getDateTime()
         for cb, series in self.__callbacks.iteritems():
@@ -322,7 +335,6 @@ class StrategyPlotter(object):
     def __init__(self, strat, plotAllInstruments=True, plotBuySell=True, plotPortfolio=True):
         self.__dateTimes = set()
 
-        self.__useAdjustedValues = strat.getUseAdjustedValues()
         self.__plotAllInstruments = plotAllInstruments
         self.__plotBuySell = plotBuySell
         self.__barSubplots = {}
@@ -374,7 +386,6 @@ class StrategyPlotter(object):
             ret = self.__barSubplots[instrument]
         except KeyError:
             ret = InstrumentSubplot(instrument, self.__plotBuySell)
-            ret.setUseAdjClose(self.__useAdjustedValues)
             self.__barSubplots[instrument] = ret
         return ret
 
