@@ -19,6 +19,7 @@
 """
 
 import abc
+import logging
 
 import pyalgotrade.broker
 from pyalgotrade.broker import backtesting
@@ -27,6 +28,7 @@ from pyalgotrade import dispatcher
 import pyalgotrade.strategy.position
 from pyalgotrade import warninghelpers
 from pyalgotrade import logger
+from pyalgotrade.barfeed import resampled
 
 
 class BaseStrategy(object):
@@ -46,23 +48,24 @@ class BaseStrategy(object):
     LOGGER_NAME = "strategy"
 
     def __init__(self, barFeed, broker):
-        self.__feed = barFeed
+        self.__barFeed = barFeed
         self.__broker = broker
         self.__activePositions = set()
         self.__orderToPosition = {}
         self.__barsProcessedEvent = observer.Event()
         self.__analyzers = []
         self.__namedAnalyzers = {}
+        self.__resampledBarFeeds = []
         self.__dispatcher = dispatcher.Dispatcher()
         self.__broker.getOrderUpdatedEvent().subscribe(self.__onOrderEvent)
-        self.__feed.getNewBarsEvent().subscribe(self.__onBars)
+        self.__barFeed.getNewBarsEvent().subscribe(self.__onBars)
 
         self.__dispatcher.getStartEvent().subscribe(self.onStart)
-        self.__dispatcher.getIdleEvent().subscribe(self.onIdle)
+        self.__dispatcher.getIdleEvent().subscribe(self.__onIdle)
 
         # It is important to dispatch broker events before feed events, specially if we're backtesting.
         self.__dispatcher.addSubject(self.__broker)
-        self.__dispatcher.addSubject(self.__feed)
+        self.__dispatcher.addSubject(self.__barFeed)
 
         # Initialize logging.
         self.__logger = logger.getLogger(BaseStrategy.LOGGER_NAME)
@@ -129,15 +132,12 @@ class BaseStrategy(object):
         ret = None
         bar = self.getFeed().getLastBar(instrument)
         if bar is not None:
-            if self.getUseAdjustedValues():
-                ret = bar.getAdjClose()
-            else:
-                ret = bar.getClose()
+            ret = bar.getPrice()
         return ret
 
     def getFeed(self):
         """Returns the :class:`pyalgotrade.barfeed.BaseBarFeed` that this strategy is using."""
-        return self.__feed
+        return self.__barFeed
 
     def getBroker(self):
         """Returns the :class:`pyalgotrade.broker.Broker` used to handle order executions."""
@@ -145,14 +145,10 @@ class BaseStrategy(object):
 
     def getCurrentDateTime(self):
         """Returns the :class:`datetime.datetime` for the current :class:`pyalgotrade.bar.Bars`."""
-        ret = None
-        bars = self.__feed.getCurrentBars()
-        if bars:
-            ret = bars.getDateTime()
-        return ret
+        return self.__barFeed.getCurrentDateTime()
 
     def marketOrder(self, instrument, quantity, onClose=False, goodTillCanceled=False, allOrNone=False):
-        """Places a market order.
+        """Submits a market order.
 
         :param instrument: Instrument identifier.
         :type instrument: string.
@@ -175,7 +171,7 @@ class BaseStrategy(object):
         if ret:
             ret.setGoodTillCanceled(goodTillCanceled)
             ret.setAllOrNone(allOrNone)
-            self.getBroker().placeOrder(ret)
+            self.getBroker().submitOrder(ret)
         return ret
 
     def order(self, instrument, quantity, onClose=False, goodTillCanceled=False, allOrNone=False):
@@ -184,7 +180,7 @@ class BaseStrategy(object):
         return self.marketOrder(instrument, quantity, onClose, goodTillCanceled, allOrNone)
 
     def limitOrder(self, instrument, limitPrice, quantity, goodTillCanceled=False, allOrNone=False):
-        """Places a limit order.
+        """Submits a limit order.
 
         :param instrument: Instrument identifier.
         :type instrument: string.
@@ -207,11 +203,11 @@ class BaseStrategy(object):
         if ret:
             ret.setGoodTillCanceled(goodTillCanceled)
             ret.setAllOrNone(allOrNone)
-            self.getBroker().placeOrder(ret)
+            self.getBroker().submitOrder(ret)
         return ret
 
     def stopOrder(self, instrument, stopPrice, quantity, goodTillCanceled=False, allOrNone=False):
-        """Places a stop order.
+        """Submits a stop order.
 
         :param instrument: Instrument identifier.
         :type instrument: string.
@@ -234,11 +230,11 @@ class BaseStrategy(object):
         if ret:
             ret.setGoodTillCanceled(goodTillCanceled)
             ret.setAllOrNone(allOrNone)
-            self.getBroker().placeOrder(ret)
+            self.getBroker().submitOrder(ret)
         return ret
 
     def stopLimitOrder(self, instrument, stopPrice, limitPrice, quantity, goodTillCanceled=False, allOrNone=False):
-        """Places a stop limit order.
+        """Submits a stop limit order.
 
         :param instrument: Instrument identifier.
         :type instrument: string.
@@ -263,7 +259,7 @@ class BaseStrategy(object):
         if ret:
             ret.setGoodTillCanceled(goodTillCanceled)
             ret.setAllOrNone(allOrNone)
-            self.getBroker().placeOrder(ret)
+            self.getBroker().submitOrder(ret)
         return ret
 
     def enterLong(self, instrument, quantity, goodTillCanceled=False, allOrNone=False):
@@ -483,19 +479,25 @@ class BaseStrategy(object):
 
     def onOrderUpdated(self, order):
         """Override (optional) to get notified when an order gets updated.
-        This is not called for orders placed using any of the enterLong or enterShort methods.
 
         :param order: The order updated.
         :type order: :class:`pyalgotrade.broker.Order`.
         """
         pass
 
+    def __onIdle(self):
+        # Force a resample check to avoid depending solely on the underlying
+        # barfeed events.
+        for resampledBarFeed in self.__resampledBarFeeds:
+            resampledBarFeed.checkNow(self.getCurrentDateTime())
+
+        self.onIdle()
+
     def __onOrderEvent(self, broker_, orderEvent):
         order = orderEvent.getOrder()
         pos = self.__orderToPosition.get(order.getId(), None)
-        if pos is None:
-            self.onOrderUpdated(order)
-        else:
+        self.onOrderUpdated(order)
+        if pos is not None:
             # Unlink the order from the position if its not active anymore.
             if not order.isActive():
                 self.unregisterPositionOrder(pos, order)
@@ -508,7 +510,7 @@ class BaseStrategy(object):
         # 1: Let analyzers process bars.
         self.__notifyAnalyzers(lambda s: s.beforeOnBars(self, bars))
 
-        # 2: Let the strategy process current bars and place orders.
+        # 2: Let the strategy process current bars and submit orders.
         self.onBars(bars)
 
         # 3: Notify that the bars were processed.
@@ -518,8 +520,8 @@ class BaseStrategy(object):
         """Call once (**and only once**) to run the strategy."""
         self.__dispatcher.run()
 
-        if self.__feed.getCurrentBars() is not None:
-            self.onFinish(self.__feed.getCurrentBars())
+        if self.__barFeed.getCurrentBars() is not None:
+            self.onFinish(self.__barFeed.getCurrentBars())
         else:
             raise Exception("Feed was empty")
 
@@ -542,6 +544,10 @@ class BaseStrategy(object):
         """Logs a message with level INFO on the strategy logger."""
         self.getLogger().info(msg)
 
+    def warning(self, msg):
+        """Logs a message with level WARNING on the strategy logger."""
+        self.getLogger().warning(msg)
+
     def error(self, msg):
         """Logs a message with level ERROR on the strategy logger."""
         self.getLogger().error(msg)
@@ -550,36 +556,62 @@ class BaseStrategy(object):
         """Logs a message with level CRITICAL on the strategy logger."""
         self.getLogger().critical(msg)
 
+    def resampleBarFeed(self, frequency, callback):
+        """
+        Builds a resampled barfeed that groups bars by a certain frequency.
+
+        :param frequency: The grouping frequency in seconds. Must be > 0.
+        :param callback: A function similar to onBars that will be called when new bars are available.
+        :rtype: :class:`pyalgotrade.barfeed.BaseBarFeed`.
+        """
+        ret = resampled.ResampledBarFeed(self.getFeed(), frequency)
+        ret.getNewBarsEvent().subscribe(callback)
+        self.getDispatcher().addSubject(ret)
+        self.__resampledBarFeeds.append(ret)
+        return ret
+
 
 class BacktestingStrategy(BaseStrategy):
     """Base class for backtesting strategies.
 
     :param barFeed: The bar feed to use to backtest the strategy.
     :type barFeed: :class:`pyalgotrade.barfeed.BaseBarFeed`.
-    :param cash: The amount of cash available.
-    :type cash: int/float.
+    :param cash_or_brk: The starting capital or a broker instance.
+    :type cash_or_brk: int/float or :class:`pyalgotrade.broker.Broker`.
 
     .. note::
         This is a base class and should not be used directly.
     """
 
-    def __init__(self, barFeed, cash=1000000):
+    def __init__(self, barFeed, cash_or_brk=1000000):
         # The broker should subscribe to barFeed events before the strategy.
-        # This is to avoid executing orders placed in the current tick.
-        broker = backtesting.Broker(cash, barFeed)
+        # This is to avoid executing orders submitted in the current tick.
+
+        if isinstance(cash_or_brk, pyalgotrade.broker.Broker):
+            broker = cash_or_brk
+        else:
+            broker = backtesting.Broker(cash_or_brk, barFeed)
+
         BaseStrategy.__init__(self, barFeed, broker)
         self.__useAdjustedValues = False
         self.setUseEventDateTimeInLogs(True)
+        self.setDebugMode(True)
 
     def getUseAdjustedValues(self):
         return self.__useAdjustedValues
 
     def setUseAdjustedValues(self, useAdjusted):
-        if not self.getFeed().barsHaveAdjClose():
-            raise Exception("The barfeed doesn't support adjusted close values")
+        self.getFeed().setUseAdjustedValues(useAdjusted)
         self.getBroker().setUseAdjustedValues(useAdjusted, True)
         self.__useAdjustedValues = useAdjusted
 
+    def setDebugMode(self, debugOn):
+        """Enable/disable debug level messages in the strategy and backtesting broker.
+        This is enabled by default."""
+        level = logging.DEBUG if debugOn else logging.INFO
+        self.getLogger().setLevel(level)
+        self.getBroker().getLogger().setLevel(level)
+
 
 class Strategy(BacktestingStrategy):
     def __init__(self, *args, **kwargs):
