@@ -22,13 +22,15 @@ from pyalgotrade.stratanalyzer import returns
 from pyalgotrade import warninghelpers
 from pyalgotrade import broker
 
+import datetime
+
 
 class PositionState(object):
     def onEnter(self, position):
         pass
 
-    # Raise an exception if an order can't be placed in the current state.
-    def canPlaceOrder(self, position, order):
+    # Raise an exception if an order can't be submitted in the current state.
+    def canSubmitOrder(self, position, order):
         raise NotImplementedError()
 
     def onOrderEvent(self, position, orderEvent):
@@ -42,7 +44,7 @@ class PositionState(object):
 
 
 class WaitingEntryState(PositionState):
-    def canPlaceOrder(self, position, order):
+    def canSubmitOrder(self, position, order):
         if position.entryActive():
             raise Exception("The entry order is still active")
 
@@ -68,8 +70,12 @@ class WaitingEntryState(PositionState):
 
 
 class OpenState(PositionState):
-    def canPlaceOrder(self, position, order):
-        # Only exit orders should be placed in this state.
+    def onEnter(self, position):
+        entryDateTime = position.getEntryOrder().getExecutionInfo().getDateTime()
+        position.setEntryDateTime(entryDateTime)
+
+    def canSubmitOrder(self, position, order):
+        # Only exit orders should be submitted in this state.
         pass
 
     def onOrderEvent(self, position, orderEvent):
@@ -101,15 +107,20 @@ class OpenState(PositionState):
         if position.entryActive():
             position.getStrategy().getBroker().cancelOrder(position.getEntryOrder())
 
-        position._placeExitOrder(stopPrice, limitPrice, goodTillCanceled)
+        position._submitExitOrder(stopPrice, limitPrice, goodTillCanceled)
 
 
 class ClosedState(PositionState):
     def onEnter(self, position):
+        # Set the exit datetime if the exit order was filled.
+        if position.exitFilled():
+            exitDateTime = position.getExitOrder().getExecutionInfo().getDateTime()
+            position.setExitDateTime(exitDateTime)
+
         assert(position.getShares() == 0)
         position.getStrategy().unregisterPosition(position)
 
-    def canPlaceOrder(self, position, order):
+    def canSubmitOrder(self, position, order):
         raise Exception("The position is closed")
 
     def onOrderEvent(self, position, orderEvent):
@@ -151,7 +162,9 @@ class Position(object):
         self.__shares = 0
         self.__strategy = strategy
         self.__entryOrder = None
+        self.__entryDateTime = None
         self.__exitOrder = None
+        self.__exitDateTime = None
         self.__posTracker = returns.PositionTracker()
         self.__allOrNone = allOrNone
 
@@ -159,21 +172,27 @@ class Position(object):
 
         entryOrder.setGoodTillCanceled(goodTillCanceled)
         entryOrder.setAllOrNone(allOrNone)
-        self.__placeAndRegisterOrder(entryOrder)
+        self.__submitAndRegisterOrder(entryOrder)
         self.__entryOrder = entryOrder
 
-    def __placeAndRegisterOrder(self, order):
+    def __submitAndRegisterOrder(self, order):
         assert(order.isInitial())
 
-        # Check if an order can be placed in the current state.
-        self.__state.canPlaceOrder(self, order)
+        # Check if an order can be submitted in the current state.
+        self.__state.canSubmitOrder(self, order)
 
-        # This may raise an exception, so we wan't to place the order before moving forward and registering the order in the strategy.
-        self.getStrategy().getBroker().placeOrder(order)
+        # This may raise an exception, so we wan't to submit the order before moving forward and registering the order in the strategy.
+        self.getStrategy().getBroker().submitOrder(order)
 
         self.__activeOrders[order.getId()] = order
         self.getStrategy().registerPositionOrder(self, order)
 
+    def setEntryDateTime(self, dateTime):
+        self.__entryDateTime = dateTime
+
+    def setExitDateTime(self, dateTime):
+        self.__exitDateTime = dateTime
+
     def switchState(self, newState):
         self.__state = newState
         self.__state.onEnter(self)
@@ -287,7 +306,7 @@ class Position(object):
             self.getStrategy().getBroker().cancelOrder(self.getExitOrder())
 
     def exitMarket(self, goodTillCanceled=None):
-        """Places a market order to close this position.
+        """Submits a market order to close this position.
 
         :param goodTillCanceled: True if the exit order is good till canceled. If False then the order gets automatically canceled when the session closes. If None, then it will match the entry order.
         :type goodTillCanceled: boolean.
@@ -301,7 +320,7 @@ class Position(object):
         self.__state.exit(self, None, None, goodTillCanceled)
 
     def exitLimit(self, limitPrice, goodTillCanceled=None):
-        """Places a limit order to close this position.
+        """Submits a limit order to close this position.
 
         :param limitPrice: The limit price.
         :type limitPrice: float.
@@ -317,7 +336,7 @@ class Position(object):
         self.__state.exit(self, None, limitPrice, goodTillCanceled)
 
     def exitStop(self, stopPrice, goodTillCanceled=None):
-        """Places a stop order to close this position.
+        """Submits a stop order to close this position.
 
         :param stopPrice: The stop price.
         :type stopPrice: float.
@@ -333,7 +352,7 @@ class Position(object):
         self.__state.exit(self, stopPrice, None, goodTillCanceled)
 
     def exitStopLimit(self, stopPrice, limitPrice, goodTillCanceled=None):
-        """Places a stop limit order to close this position.
+        """Submits a stop limit order to close this position.
 
         :param stopPrice: The stop price.
         :type stopPrice: float.
@@ -363,7 +382,7 @@ class Position(object):
 
         self.__state.exit(self, stopPrice, limitPrice, goodTillCanceled)
 
-    def _placeExitOrder(self, stopPrice, limitPrice, goodTillCanceled):
+    def _submitExitOrder(self, stopPrice, limitPrice, goodTillCanceled):
         assert(not self.exitActive())
 
         exitOrder = self.buildExitOrder(stopPrice, limitPrice)
@@ -375,7 +394,7 @@ class Position(object):
 
         exitOrder.setAllOrNone(self.__allOrNone)
 
-        self.__placeAndRegisterOrder(exitOrder)
+        self.__submitAndRegisterOrder(exitOrder)
         self.__exitOrder = exitOrder
 
     def onOrderEvent(self, orderEvent):
@@ -388,10 +407,11 @@ class Position(object):
         # Update the number of shares.
         if orderEvent.getEventType() in (broker.OrderEvent.Type.PARTIALLY_FILLED, broker.OrderEvent.Type.FILLED):
             execInfo = orderEvent.getEventInfo()
+            # roundQuantity is used to prevent bugs like the one triggered in testcases.bitstamp_test:TestCase.testRoundingBug
             if order.isBuy():
-                self.__shares += execInfo.getQuantity()
+                self.__shares = order.getInstrumentTraits().roundQuantity(self.__shares + execInfo.getQuantity())
             else:
-                self.__shares -= execInfo.getQuantity()
+                self.__shares = order.getInstrumentTraits().roundQuantity(self.__shares - execInfo.getQuantity())
 
         self.__state.onOrderEvent(self, orderEvent)
 
@@ -411,6 +431,24 @@ class Position(object):
         """Returns True if the position is open."""
         return self.__state.isOpen(self)
 
+    def getAge(self):
+        """Returns the duration in open state.
+
+        :rtype: datetime.timedelta.
+
+        .. note::
+            * If the position is open, then the difference between the entry datetime and the datetime of the last bar is returned.
+            * If the position is closed, then the difference between the entry datetime and the exit datetime is returned.
+        """
+        ret = datetime.timedelta()
+        if self.__entryDateTime is not None:
+            if self.__exitDateTime is not None:
+                last = self.__exitDateTime
+            else:
+                last = self.__strategy.getCurrentDateTime()
+            ret = last - self.__entryDateTime
+        return ret
+
 
 # This class is reponsible for order management in long positions.
 class LongPosition(Position):
@@ -430,6 +468,7 @@ class LongPosition(Position):
 
     def buildExitOrder(self, stopPrice, limitPrice):
         quantity = self.getShares()
+        assert(quantity > 0)
         if limitPrice is None and stopPrice is None:
             ret = self.getStrategy().getBroker().createMarketOrder(broker.Order.Action.SELL, self.getInstrument(), quantity, False)
         elif limitPrice is not None and stopPrice is None:
@@ -462,6 +501,7 @@ class ShortPosition(Position):
 
     def buildExitOrder(self, stopPrice, limitPrice):
         quantity = self.getShares() * -1
+        assert(quantity > 0)
         if limitPrice is None and stopPrice is None:
             ret = self.getStrategy().getBroker().createMarketOrder(broker.Order.Action.BUY_TO_COVER, self.getInstrument(), quantity, False)
         elif limitPrice is not None and stopPrice is None:
