@@ -35,3 +35,12 @@ def safe_min(left, right):
         return left
     else:
         return min(left, right)
+
+
+def safe_max(left, right):
+    if left is None:
+        return right
+    elif right is None:
+        return left
+    else:
+        return max(left, right)
